using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using WAF.Presentation.Web;
using WAF.Common;
using WAF.Engine;
using WAF.Engine.Query;
using WAF.Engine.Content.Native;
using System.Text;
using WAF.Resources;
using System.Collections.Generic;
using System.Globalization;

public partial class WAF_Login : System.Web.UI.Page {

    const string prefUILangCookieName = "waf-ui-pref-lang";

    protected void Page_Load(object sender, EventArgs e) {

        if (Request["RelKeyModule"] != null) {
            this.loginNewUI(Request["RelKeyModule"]);
        }

        chkAnonymous.Visible = WAFContext.AllowAnonymousEditLogin;
        imgLogo.ImageUrl = WAFContext.UrlFromRootToApp + "WAF/Edit/Main/Images/logo.png";
        StringBuilder tooltip = new StringBuilder();
        tooltip.AppendLine();
        tooltip.AppendLine("    " + WAFRuntime.Engine.License.BrandingProductName + "    ");
        tooltip.AppendLine();
        tooltip.Append("      Nativ build: " + WAFRuntime.Engine.NativeDefinition.Build);
        tooltip.AppendLine(" - Local build: " + WAFRuntime.Engine.LocalDefinition.Build + "    ");
        tooltip.AppendLine(" ");
        imgLogo.ToolTip = tooltip.ToString();
        WAF_Edit_Standard master = (WAF_Edit_Standard)this.Master;
        master.EditModule.DeltaWindowHeight = -1;
        if (this.chkAnonymous.Checked) {
            chkAnonymous.Focus();
        } else {
            txtUserName.Focus();
        }
        if (!Page.IsPostBack) {
            switch ((string)Request["Action"]) {
                case "LogOff":
                    WAFContext.LogOffEdit();
                    int selectedLang = GetCookiePreferredLanguage();
                    if (selectedLang > 0) {
                        WAFContext.Session.UILCID = selectedLang;
                    }
                    lblStatus.Text = HttpUtility.HtmlEncode(Local.Text("Web.WAF.Edit.Login.LoggedOff"));
                    break;
                default:
                    break;
            }
        }

    }
    void storeDomainAtEditLogin() {
        Uri uri = Request.Url;
        Session["WAF_DomainAtEditLogin"] = WAFContext.UrlFromRootToApp;
    }

    protected override void OnPreRender(EventArgs e) {
        int selectedLang = 0;
        if (!IsPostBack) {
            selectedLang = GetCookiePreferredLanguage();
            if (selectedLang == 0) {
                selectedLang = WAFContext.Session.UILCID;
            } else {
                WAFContext.Session.UILCID = selectedLang;
            }
        } else {
            selectedLang = Utils.GetIntegerOnly(ddlLanguages.SelectedValue);
        }
        lblPleaseLogIn.Text = HttpUtility.HtmlEncode(Local.Text("Web.WAF.Edit.Main.PleaseLogin"));
        lblUserName.Text = Local.Text("Web.WAF.Edit.Main.LoginUserName");
        lblPassword.Text = Local.Text("Web.WAF.Edit.Main.LoginPassword");
        lblSMSCode.Text = Local.Text("Web.WAF.Edit.Login.SMSCode");
        btnLogin.Text = Local.Text("Web.WAF.Edit.Main.LoginButtonLogIn");
        lblLanguage.Text = Local.Text("Web.WAF.Edit.Main.LoginUILanguageLabel");
        chkAnonymous.Text = Local.Text("Web.WAF.Edit.Main.LoginCheckBoxAnonymous");
        List<int> uiCultures = Local.GetUICultures();

        ddlLanguages.Items.Clear();
        foreach (int lcid in uiCultures) {
            CultureInfo culture = CultureInfo.GetCultureInfo(lcid);
            ddlLanguages.Items.Add(new ListItem(ToTitleCase(culture.NativeName.Substring(0, culture.NativeName.IndexOf("("))), lcid.ToString()));
        }
        ddlLanguages.SelectedValue = selectedLang.ToString();
        base.OnPreRender(e);
    }
    public string ToTitleCase(string str) {
        return CultureInfo.CurrentCulture.TextInfo.ToTitleCase(str.ToLower());
    }
    void loginNewUI(string token) {
        var parts = token.Split('|');
        if (parts.Length < 2) return;
        var d = new DateTime(long.Parse(parts[1]));
        var ageInSec = DateTime.Now.Subtract(d).TotalSeconds;
        if (ageInSec > 10) return;
        foreach (var s in WAFRuntime.Engine.GetAllSessions()) {
            if (s.LoginToken == token) {
                WAFSession newSession = WAFContext.LogIn(s, true);
                s.LoginToken = null;
                string returnUrl = Request["ReturnUrl"];
                Response.Redirect("/edit", false);
                Context.ApplicationInstance.CompleteRequest();
            }
        }
    }
    protected void btnLogin_Click(object sender, EventArgs e) {
        string returnUrl = Request["ReturnUrl"];
        if (returnUrl != null && !returnUrl.StartsWith(WAFContext.GetUrlFromRootToHost(true))) returnUrl = null;
        if (returnUrl != null) {
            var startQ = returnUrl.IndexOf("?");
            if (startQ > -1) {
                var unFiltered = HttpUtility.ParseQueryString(returnUrl.Substring(startQ));
                var filtered = string.Empty;
                var nid = Utils.GetLongOnly(unFiltered["nid"]);
                if (nid > 0) filtered += "&nid=" + nid;
                var lcid = Utils.GetLongOnly(unFiltered["lcid"]);
                if (lcid > 0) filtered += "&lcid=" + lcid;
                var rev = Utils.GetLongOnly(unFiltered["rev"]);
                if (rev > 0) filtered += "&rev=" + rev;
                var open = CKeyNLR.TryParse(unFiltered["Open"]);
                if (open != null) filtered += "&Open=" + open.ToString();
                returnUrl = returnUrl.Substring(0, startQ) + (filtered.Length > 0 ? "?" + filtered.Substring(1) : null);
            }
        }
        if (returnUrl == null) returnUrl = WAFContext.UrlFromHostToFolderPhysicalEdit + WAFContext.GetEditStartUpModule();
        try {
            if (chkAnonymous.Checked) {
                WAFContext.LogInEditAnonymous();
                storeDomainAtEditLogin();
                Response.Redirect(returnUrl);
            } else {
                int userId = 0;
                AutenticationFailureReason failureReason;
                var correctCode = !string.IsNullOrWhiteSpace(txtSMScode.Text) && (Session["TwoFactorAuth"] + "") == txtSMScode.Text && (Session["TwoFactorAuthUser"] + "") == txtUserName.Text;
                if (correctCode || WAFContext.Engine.AutenticateUser(txtUserName.Text, txtPassword.Text, out userId, out failureReason)) {
                    if (WAFRuntime.Installation.TwoFactorAuthentication(txtUserName.Text) && !correctCode) {
                        string mobile;
                        string email;
                        if (userId > 0) {
                            var user = WAFContext.Engine.SystemSession.GetContent<SystemUser>(userId);
                            mobile = user.Mobile;
                            email = user.Email;
                        } else {
                            mobile = WAFRuntime.Installation.AdminMobile;
                            email = WAFRuntime.Installation.AdminEmail;
                        }
                        var newCode = new Random().Next(99999).ToString("00000");
                        Session["TwoFactorAuth"] = newCode;
                        Session["TwoFactorAuthUser"] = txtUserName.Text;
                        if (!string.IsNullOrWhiteSpace(mobile)) {
                            try {
                                WAFContext.Session.SendSms(newCode, mobile, Request.Url.Host, false);
                                lblStatus.Text = HttpUtility.HtmlEncode(Local.Text("Web.WAF.Edit.Login.EnterSMSCodeSent") + " " + mobile + ":");
                            } catch { // fallback in case SMS is not working ( could be license restriction )
                                WAFContext.Session.SendEmail(true, "Code: " + newCode, email, WAFRuntime.Installation.DefaultFromEmail, "Two factor verification code");
                                lblStatus.Text = HttpUtility.HtmlEncode("Enter code sent to " + " " + email + ":");
                            }
                            trUserName.Visible = false;
                            trPassword.Visible = false;
                            trSMSCode.Visible = true;
                            txtSMScode.Focus();
                            return;
                        } else if (!string.IsNullOrWhiteSpace(email)) {
                            WAFContext.Session.SendEmail(true, "Code: " + newCode, email, WAFRuntime.Installation.DefaultFromEmail, "Two factor verification code");
                            lblStatus.Text = HttpUtility.HtmlEncode("Enter code sent to " + " " + email + ":");
                            trUserName.Visible = false;
                            trPassword.Visible = false;
                            trSMSCode.Visible = true;
                            txtSMScode.Focus();
                            return;
                        } else {
                            lblStatus.Text = HttpUtility.HtmlEncode(Local.Text("Web.WAF.Edit.Login.2AuthFailed"));
                            return;
                        }
                    }
                    WAFSession newSession = WAFContext.LogInEdit(txtUserName.Text);
                    int prefLang = Utils.GetIntegerOnly(ddlLanguages.SelectedValue);
                    if (newSession.UILCID != prefLang && prefLang > 0) {
                        SetCookiePreferredLanguage(prefLang);
                        //if user is stored, update preferred UI language on user
                        if (newSession.Access.IsStored) {
                            SystemUser user = newSession.GetUser();
                            user.PreferredUILanguage = prefLang;
                            user.UpdateChanges();
                        }
                    }
                    newSession.UILCID = prefLang;
                    newSession.SiteId = newSession.GetDefaultSiteId();
                    storeDomainAtEditLogin();
                    Response.Redirect(returnUrl, false);
                    Context.ApplicationInstance.CompleteRequest();
                } else {
                    switch (failureReason) {
                        case AutenticationFailureReason.None: break;
                        case AutenticationFailureReason.UsernameIsNotUnique: lblStatus.Text = HttpUtility.HtmlEncode(Local.Text("Web.WAF.Edit.Login.UsernameIsNotUnique")); break;
                        case AutenticationFailureReason.UserIsLocked: lblStatus.Text = HttpUtility.HtmlEncode(Local.Text("Web.WAF.Edit.Login.UserIsLocked")); break;
                        case AutenticationFailureReason.UserHostAddressIsBlocked: lblStatus.Text = HttpUtility.HtmlEncode("Web.WAF.Edit.Login.UserHostAddressIsBlocked"); break;
                        case AutenticationFailureReason.Other: lblStatus.Text = HttpUtility.HtmlEncode(Local.Text("Web.WAF.Edit.Login.Other")); break;
                        case AutenticationFailureReason.UsernameIsUnknown:
                        case AutenticationFailureReason.PasswordIsInvalid:
                        default:
                            lblStatus.Text = HttpUtility.HtmlEncode(Local.Text("Web.WAF.Edit.Login.InvalidCredentials"));
                            break;
                    }
                }
            }
        } catch {
            lblStatus.Text = HttpUtility.HtmlEncode(Local.Text("Web.WAF.Edit.Login.Failed"));
        }
    }

    private void SetCookiePreferredLanguage(int prefLang) {
        HttpContext context = HttpContext.Current;
        HttpCookie cookie;
        cookie = new HttpCookie(prefUILangCookieName, prefLang.ToString());
        cookie.HttpOnly = true;
        context.Response.SetCookie(cookie);
    }
    private int GetCookiePreferredLanguage() {
        HttpCookie prefUILangCookie = new HttpCookie(prefUILangCookieName);
        prefUILangCookie = Request.Cookies[prefUILangCookieName];
        if (prefUILangCookie != null) {
            try {
                return Utils.GetIntegerOnly(prefUILangCookie.Value.ToString());
            } catch { }
        }
        return 0;

    }
    protected void chkAnonymous_CheckedChanged(object sender, EventArgs e) {
        if (chkAnonymous.Checked) btnLogin_Click(sender, e);
        txtUserName.Enabled = !chkAnonymous.Checked;
        txtPassword.Enabled = !chkAnonymous.Checked;
    }
    protected void ddlLanguages_SelectedIndexChanged(object sender, EventArgs e) {
        WAFContext.Session.UILCID = Utils.GetIntegerOnly(ddlLanguages.SelectedValue);
        SetCookiePreferredLanguage(WAFContext.Session.UILCID);
    }
}
